#!/bin/bash

## TODO
# Day 2 operations
# provisionning
# IPs
# Group
# Resize 
# Extra Params

## Helper
# ./script.sh bigsur allOS 
OSX_VERSION=$1
DEVICE_OS_RANGE=$2

sec_option=''

## Target VM Name (exportable ?)
__IQ_ROOT_TICKER='iq'
__IQ_ROOT_ENV='auto'

## Target VM Specs (yaml)
__VM_VCPU_COUNT=4 #2
__VM_VCPU_EXEC_CAP=100
__VM_MEM_SIZE=8192 #4096
__HPVZRGROUP_NAME='IQ\ Qualification\ Pool'

## VMB env
vbroot='/Users/admin/vbox/'
vmd=$(echo $vbroot'vms/')
vbm='/usr/local/bin/VBoxManage'
srp='/Users/admin/usr/vlu'
log=$(echo $srp'/logs/provisioning_broker.log')
vmlg=$(echo $vbroot'vm_ico/')

## Templating
TEMPLATE_BASE_NAME='template'
PG_GROUP='provisionning'
PG_GROUP_FOLDER='Provisionning Group/'
VM_BASE_PATH='/Users/admin/vbox/vms/'

#binding: template-bigsur-provisionning_allOS
TEMPLATE_NAME=$(echo $TEMPLATE_BASE_NAME'-'$OSX_VERSION'_'$PG_GROUP'_'$DEVICE_OS_RANGE)
TEMPLATE_PATH=$(echo $VM_BASE_PATH$PG_GROUP_FOLDER)
TEMPLATE_FPWD=$(echo $TEMPLATE_PATH$TEMPLATE_NAME'/')

#echo 'FPWD: '$TEMPLATE_FPWD

function logger () {
	__date=$(date "+%Y-%m-%d %H:%M:%S")
	__logMsg=$(echo -e $__date'\n'$@)

	if [[ ${__logMsg: -2} != '\n' ]] && [[ ${__logMsg: -2} != '\t' ]]; then __logMsg=$(echo $__logMsg'\n'); fi

	echo -ne $__logMsg >> $log
	if [[ $sec_option != 'silence' ]]; then echo -ne $__logMsg;fi
}

function stopvm () {
	eval $vbm controlvm $1 poweroff
}

function resetvm () {
	eval $vbm controlvm $1 reset
}

function startvm () {
	eval $vbm startvm $1 --type headless
}

#function cloneTemplate () {
#	evbal 
#}

function getOSSpecs () {
	__osRangeLC=$(echo "$DEVICE_OS_RANGE" | awk '{print tolower($0)}')

	case $__osRangeLC in
		ios* )
			__VM_ICONFILE=$(echo $vmlg'ios.png');
			;;
		and* )
			__VM_ICONFILE=$(echo $vmlg'android.png');
			;;
		web* )
			__VM_ICONFILE=$(echo $vmlg'web.png');
			;;
		all* )
			__VM_ICONFILE=$(echo $vmlg'allos.png');
			;;
		* )
			echo "no match!"
			;;
	esac

	__vmNICMacAddr=$(cat $srp'/etc/iq_test/macAddrRelationTable' | grep -wi "${__vmName}" | head -n 1 | awk '{print $1}')

	if [[ __vmNICMacAddr == '' ]]; then
		logger "Error: No mac Address found for vmName=${__vmName}"
		exit 1;
	else
		logger "Found Mac: ${__vmNICMacAddr} for VM ${__vmName}"
	fi

}

function provisionvm () {
	#mainframe booting started
	#pull out
	# icon
	# memory
	# cpu cap
	# custom mac address & add router
	# post install : deploy ssh keys ?
	# Custom mac and new VMUID
	# #cat /Users/admin/usr/vlu/etc/iq_test/macAddrRelationTable | grep $1 | head -n 1 | awk '{print $1}'
	__vmName=$1
	getOSSpecs

	logger "Cloning ${TEMPLATE_NAME} --> ${__vmName} \t"
	eval $vbm clonevm $TEMPLATE_NAME --name="${__vmName}" --register --groups="'/'${__HPVZRGROUP_NAME}" --mode=all --options=keephwuuids #--options=keepdisknames #--options=keepallmacs

	logger "Pull out VM ${__vmName} --> ${__HPVZRGROUP_NAME}/${__vmName}"
	eval $vbm modifyvm $__vmName --groups '/'${__HPVZRGROUP_NAME}
	eval $vbm unregistervm $__vmName

	if [[ -d "${vmd}${__HPVZRGROUP_NAME}'/'${__vmName}" ]]; then
		logger "VM ${__vmName} is positionned within the correct pool/group."
	else
		eval mv "${vmd}${__vmName}" "${vmd}${__HPVZRGROUP_NAME}'/'${__vmName}"
	fi

	logger "Push back VM ${__vmName}"
	eval $vbm registervm $vmd$__HPVZRGROUP_NAME'/'$__vmName'/'$__vmName'.vbox'
	eval $vbm modifyvm $__vmName --iconfile ${__VM_ICONFILE}	

	logger "Setting up VM ${__vmName} with: ${__VM_VCPU_COUNT} vCPUs, ${__VM_MEM_SIZE}Mb RAM, ${__VM_VCPU_EXEC_CAP}% vCPU cap. and ${__vmNICMacAddr} as MacAddr for NIC1"
	eval $vbm modifyvm $__vmName --macaddress1 ${__vmNICMacAddr}
	eval $vbm modifyvm $__vmName --cpus ${__VM_VCPU_COUNT}
	eval $vbm modifyvm $__vmName --memory ${__VM_MEM_SIZE}
	eval $vbm modifyvm $__vmName --cpuexecutioncap ${__VM_VCPU_EXEC_CAP}

	logger "${__vmName} Mainframe booting Initiated"
	startvm $__vmName
}

__allowed=false
#################################
## Checking if template exists ##
if [[ -d "${TEMPLATE_FPWD}" ]]; then
	__allowed=true
	TEMPLATE_NAME=$(ls "$TEMPLATE_PATH" | grep -i $TEMPLATE_NAME)
	echo "Template [${TEMPLATE_NAME}] exists on your filesystem."
fi

if [[ $__allowed != true ]]; then
	echo "${BASH_SOURCE##*/} - Error: Template not found"
else
	## Exec
	# Check existing VMs 
	# Clone template towards : iq-auto-osxEnv-deviceOS
	# Resize if provisionning
	# Join group

	__desiredVMName=$(echo $__IQ_ROOT_TICKER'-'$__IQ_ROOT_ENV'-'$OSX_VERSION'-'$DEVICE_OS_RANGE)	
	__isVMAlreadyExists=$(eval $vbm list vms | grep -ci "${__desiredVMName}")
	__isVMAlreadyPowered=$(eval $vbm list runningvms | grep -ci "${__desiredVMName}")

	if [[ $__isVMAlreadyExists -gt 0 ]]; then
		#Present within list
		echo 'VM exists'

		if [[ $__isVMAlreadyPowered -gt 0 ]]; then
			#is powered
			echo 'VM is powered'
		else
			#exists but is not powered --> Power VM
			echo 'VM is not powered : Starting up!'
			startvm $__desiredVMName
		fi
	else
		# does not exists --> Need to provision it
		provisionvm $__desiredVMName
	fi
fi